@@ -4,6 +4,7 @@ import logging as log
 import math
 from collections.abc import Generator
 from dataclasses import dataclass
+from functools import cached_property
 from pathlib import Path
 
 import h5py
@@ -11,7 +12,7 @@ import numpy as np
 
 from ftag.cuts import Cuts
 from ftag.hdf5.h5utils import get_dtype
-from ftag.vds import create_virtual_file
+from ftag.sample import Sample
 
 
 @dataclass
@@ -21,13 +22,13 @@ class H5SingleReader:
     jets_name: str = "jets"
     precision: str | None = None
     shuffle: bool = True
+    do_remove_inf: bool = False
 
     def __post_init__(self) -> None:
-        self.fname = str(self.fname)
-        if "*" in self.fname:
-            self.fname = create_virtual_file(self.fname)
+        self.sample = Sample(self.fname)
+        self.fname = self.sample.virtual_file()
 
-    @property
+    @cached_property
     def num_jets(self) -> int:
         with h5py.File(self.fname) as f:
             return len(f[self.jets_name])
@@ -74,13 +75,14 @@ class H5SingleReader:
             variables = {self.jets_name: None}
 
         total = 0
+        rng = np.random.default_rng(42)
         with h5py.File(self.fname) as f:
             data = {name: self.empty(f[name], var) for name, var in variables.items()}
 
             # get indices
             indices = list(range(0, self.num_jets, self.batch_size))
             if self.shuffle:
-                np.random.default_rng(42).shuffle(indices)
+                rng.shuffle(indices)
 
             # loop over batches and read file
             for low in indices:
@@ -93,7 +95,8 @@ class H5SingleReader:
                     data = {name: array[idx] for name, array in data.items()}
 
                 # check for inf and remove
-                data = self.remove_inf(data)
+                if self.do_remove_inf:
+                    data = self.remove_inf(data)
 
                 # check for completion
                 total += len(data[self.jets_name])
@@ -108,12 +111,33 @@ class H5SingleReader:
 
 @dataclass
 class H5Reader:
+    """Reads data from multiple HDF5 files.
+
+    Parameters
+    ----------
+    fname : Path | str | list[Path | str]
+        Path to the HDF5 file or list of paths
+    batch_size : int, optional
+        Number of jets to read at a time, by default 100_000
+    jets_name : str, optional
+        Name of the jets dataset, by default "jets"
+    precision : str | None, optional
+        Cast floats to given precision, by default None
+    shuffle : bool, optional
+        Read batches in a shuffled order, by default True
+    weights : list[float] | None, optional
+        Weights for different input datasets, by default None
+    do_remove_inf : bool, optional
+        Remove jets with inf values, by default False
+    """
+
     fname: Path | str | list[Path | str]
     batch_size: int = 100_000
     jets_name: str = "jets"
     precision: str | None = None
     shuffle: bool = True
     weights: list[float] | None = None
+    do_remove_inf: bool = False
 
     def __post_init__(self) -> None:
         if isinstance(self.fname, (str, Path)):
@@ -126,23 +150,25 @@ class H5Reader:
 
         # create readers
         self.readers = [
-            H5SingleReader(fname, batch_size, self.jets_name, self.precision, self.shuffle)
-            for fname, batch_size in zip(self.fname, self.batch_sizes)
+            H5SingleReader(f, b, self.jets_name, self.precision, self.shuffle, self.do_remove_inf)
+            for f, b in zip(self.fname, self.batch_sizes)
         ]
 
-    @property
-    def vds_path(self) -> Path:
-        # TODO: this is mostly needed to get the src dtype,
-        # there is probably a better way of exposing this directly
-        return Path(self.readers[0].fname)
-
     @property
     def num_jets(self) -> int:
         return sum(r.num_jets for r in self.readers)
 
-    def get_dtype(self, name: str) -> np.dtype:
-        with h5py.File(self.readers[0].fname) as f:
-            return f[name].dtype
+    @property
+    def files(self) -> list[Path]:
+        return [Path(r.fname) for r in self.readers]
+
+    @cached_property
+    def dtypes(self) -> dict[str, np.dtype]:
+        dtypes = {}
+        with h5py.File(self.files[0]) as f:
+            for key in f:
+                dtypes[key] = f[key].dtype
+        return dtypes
 
     def stream(
         self, variables: dict | None = None, num_jets: int | None = None, cuts: Cuts | None = None
