@@ -30,8 +30,9 @@ class H5Writer:
         self.dst.parent.mkdir(parents=True, exist_ok=True)
         self.file = h5py.File(self.dst, "w")
         self.add_attr("srcfile", str(self.src))
-        self.git_hash = check_output(["git", "rev-parse", "HEAD"]).decode("ascii").strip()
-        self.add_attr("git_hash", self.git_hash)
+        git_hash = check_output(["git", "rev-parse", "--short", "HEAD"], cwd=Path(__file__).parent)
+        self.git_hash = git_hash.decode("ascii").strip()
+        self.add_attr("writer_hash", self.git_hash)
         for name, var in self.variables.items():
             self.create_ds(name, var)
 
