@@ -33,7 +33,9 @@ def get_virtual_layout(fnames: list[str], group: str):
     return layout
 
 
-def create_virtual_file(pattern: str, out_fname: Path | None = None, overwrite: bool = False):
+def create_virtual_file(
+    pattern: Path | str, out_fname: Path | None = None, overwrite: bool = False
+):
     # get list of filenames
     fnames = glob.glob(str(pattern))
     if not fnames:
@@ -43,7 +45,10 @@ def create_virtual_file(pattern: str, out_fname: Path | None = None, overwrite:
     if out_fname is None:
         assert len(set(Path(fname).parent for fname in fnames)) == 1
         out_fname = Path(fnames[0]).parent / "vds" / "vds.h5"
+    else:
+        out_fname = Path(out_fname)
 
+    # check if file already exists
     if not overwrite and out_fname.is_file():
         return out_fname
 
@@ -55,3 +60,24 @@ def create_virtual_file(pattern: str, out_fname: Path | None = None, overwrite:
             f.create_virtual_dataset(group, layout)
 
     return out_fname
+
+
+def main():
+    import argparse
+
+    parser = argparse.ArgumentParser(description="Create a virtual dataset from a glob pattern")
+    parser.add_argument("pattern", type=Path, help="Glob pattern of files to merge")
+    parser.add_argument("output", type=Path, help="Output file name")
+    args = parser.parse_args()
+
+    print(f"Globbing {args.pattern}...")
+    create_virtual_file(args.pattern, args.output, overwrite=True)
+    with h5py.File(args.output) as f:
+        key = list(f.keys())[0]
+        num = len(f[key])
+    print(f"Virtual dataset '{key}' has {num:,} entries")
+    print(f"Saved virtual file to {args.output.resolve()}")
+
+
+if __name__ == "__main__":
+    main()
